@@ -49,21 +49,21 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* VOID:INT,STRING,POINTER (./server-marshal.list:1) */
+/* VOID:INT,STRING,VARIANT (./server-marshal.list:1) */
 void
-_dbusmenu_server_marshal_VOID__INT_STRING_POINTER (GClosure     *closure,
+_dbusmenu_server_marshal_VOID__INT_STRING_VARIANT (GClosure     *closure,
                                                    GValue       *return_value G_GNUC_UNUSED,
                                                    guint         n_param_values,
                                                    const GValue *param_values,
                                                    gpointer      invocation_hint G_GNUC_UNUSED,
                                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__INT_STRING_POINTER) (gpointer     data1,
+  typedef void (*GMarshalFunc_VOID__INT_STRING_VARIANT) (gpointer     data1,
                                                          gint         arg_1,
                                                          gpointer     arg_2,
                                                          gpointer     arg_3,
                                                          gpointer     data2);
-  register GMarshalFunc_VOID__INT_STRING_POINTER callback;
+  register GMarshalFunc_VOID__INT_STRING_VARIANT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -79,12 +79,12 @@ _dbusmenu_server_marshal_VOID__INT_STRING_POINTER (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__INT_STRING_POINTER) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__INT_STRING_VARIANT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_marshal_value_peek_int (param_values + 1),
             g_marshal_value_peek_string (param_values + 2),
-            g_marshal_value_peek_pointer (param_values + 3),
+            g_marshal_value_peek_variant (param_values + 3),
             data2);
 }
 
